@@ -634,6 +634,118 @@ TOPLEVEL_GET_SET_FUNCS ("background/fit",     background, bool, fit,
 TOPLEVEL_GET_SET_FUNCS ("background/stretch", background, bool, stretch,        gboolean)
 TOPLEVEL_GET_SET_FUNCS ("background/rotate",  background, bool, rotate,         gboolean)
 
+static const char *
+panel_profile_get_attached_object_key (PanelToplevel *toplevel,
+				       const char    *key)
+{
+	GtkWidget  *attach_widget;
+	const char *id;
+
+	attach_widget = panel_toplevel_get_attach_widget (toplevel);
+
+	id = panel_applet_get_id_by_widget (attach_widget);
+
+	if (!id)
+		return NULL;
+
+	return panel_gconf_full_key (PANEL_GCONF_OBJECTS, current_profile, id, key);
+}
+
+void
+panel_profile_set_attached_custom_icon (PanelToplevel *toplevel,
+					const char    *custom_icon)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "use_custom_icon");
+	if (key)
+		gconf_client_set_bool (client, key, custom_icon != NULL, NULL);
+
+	key = panel_profile_get_attached_object_key (toplevel, "custom_icon");
+	if (key)
+		gconf_client_set_string (client, key, custom_icon, NULL);
+}
+
+char *
+panel_profile_get_attached_custom_icon (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "use_custom_icon");
+	if (!key || !gconf_client_get_bool (client, key, NULL))
+		return NULL;
+
+	key = panel_profile_get_attached_object_key (toplevel, "custom_icon");
+
+	return key ? gconf_client_get_string (client, key, NULL) : NULL;
+}
+
+gboolean
+panel_profile_is_writable_attached_custom_icon (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "use_custom_icon");
+	if (!key)
+		return TRUE;
+
+	if (!gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	key = panel_profile_get_attached_object_key (toplevel, "custom_icon");
+
+	return key ? gconf_client_key_is_writable (client, key, NULL) : TRUE;
+}
+
+void
+panel_profile_set_attached_tooltip (PanelToplevel *toplevel,
+				    const char    *tooltip)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "tooltip");
+	if (key)
+		gconf_client_set_string (client, key, tooltip, NULL);
+}
+
+char *
+panel_profile_get_attached_tooltip (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "tooltip");
+
+	return key ? gconf_client_get_string (client, key, NULL) : NULL;
+}
+
+gboolean
+panel_profile_is_writable_attached_tooltip (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "tooltip");
+
+	return key ? gconf_client_key_is_writable (client, key, NULL) : TRUE;
+}
+
 static PanelBackgroundType
 get_background_type (GConfClient *client,
 		     const char  *toplevel_dir)
@@ -1332,17 +1444,23 @@ panel_profile_load_toplevel (GConfClient       *client,
 		panel_gconf_associate_schemas_in_dir (
 			client, toplevel_dir, PANEL_SCHEMAS_DIR "/toplevels");
 
+	gconf_client_add_dir (client,
+			      toplevel_dir,
+			      GCONF_CLIENT_PRELOAD_ONELEVEL,
+			      NULL);
+
+	key = panel_gconf_sprintf ("%s/background", toplevel_dir);
+	gconf_client_add_dir (client,
+			      key,
+			      GCONF_CLIENT_PRELOAD_ONELEVEL,
+			      NULL);
+
 	if (!(screen = get_toplevel_screen (client, toplevel_dir))) {
 		g_free (toplevel_id);
 		g_free (toplevel_dir);
 		return NULL;
 	}
 
-	gconf_client_add_dir (client,
-			      toplevel_dir,
-			      GCONF_CLIENT_PRELOAD_RECURSIVE,
-			      NULL);
-
 	toplevel = g_object_new (PANEL_TYPE_TOPLEVEL,
 				 "screen", screen,
 				 NULL);
@@ -1559,6 +1677,7 @@ panel_profile_load_object (GConfClient       *client,
 			   char              *id)
 {
 	PanelObjectType  object_type;
+	char            *object_dir;
 	const char      *key;
 	char            *type_string;
 	char            *toplevel_id;
@@ -1566,7 +1685,14 @@ panel_profile_load_object (GConfClient       *client,
 	gboolean         right_stick;
 	gboolean         locked;
 
-	key = panel_gconf_full_key (type, current_profile, id, "object_type");
+	object_dir = g_strdup_printf ("%s/%s/%s",
+				      profile_dir,
+				      type == PANEL_GCONF_OBJECTS ? "objects" : "applets",
+				      id);
+
+	gconf_client_add_dir (client, object_dir, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
+	key = panel_gconf_sprintf ("%s/object_type", object_dir);
 	type_string = gconf_client_get_string (client, key, NULL);
         
 	if (!panel_profile_map_object_type_string (type_string, &object_type)) {
@@ -1576,18 +1702,20 @@ panel_profile_load_object (GConfClient       *client,
 	
 	g_free (type_string);
 
-	key = panel_gconf_full_key (type, current_profile, id, "position");
+	key = panel_gconf_sprintf ("%s/position", object_dir);
 	position = gconf_client_get_int (client, key, NULL);
 	
-	key = panel_gconf_full_key (type, current_profile, id, "toplevel_id");
+	key = panel_gconf_sprintf ("%s/toplevel_id", object_dir);
 	toplevel_id = gconf_client_get_string (client, key, NULL);
 
-	key = panel_gconf_full_key (type, current_profile, id, "panel_right_stick");
+	key = panel_gconf_sprintf ("%s/panel_right_stick", object_dir);
 	right_stick = gconf_client_get_bool (client, key, NULL);
 
-	key = panel_gconf_full_key (type, current_profile, id, "locked");
+	key = panel_gconf_sprintf ("%s/locked", object_dir);
 	locked = gconf_client_get_bool (client, key, NULL);
 
+	g_free (object_dir);
+
 	panel_applet_queue_applet_to_load (id,
 					   object_type,
 					   toplevel_id,
@@ -1703,14 +1831,14 @@ panel_profile_load_added_ids (GConfClient            *client,
 	}
 
 	for (l = added_ids; l; l = l->next) {
-		char *toplevel_id = l->data;
+		char *id = l->data;
 
 		if (!profile_dir)
 			profile_dir = gconf_concat_dir_and_key (
-						PANEL_CONFIG_DIR, current_profile);
+				PANEL_CONFIG_DIR, current_profile);
 
-		/* takes ownership of toplevel_id */
-		load_handler (client, profile_dir, type, toplevel_id);
+		/* takes ownership of id */
+		load_handler (client, profile_dir, type, id);
 	}
 
 	g_slist_free (added_ids);
@@ -1755,7 +1883,7 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 				       GConfEntry  *entry)
 {
 	GConfValue *value;
-	GSList     *existing_toplevels;
+	GSList     *l, *existing_toplevels;
 	GSList     *toplevel_ids;
 
 	if (!(value = gconf_entry_get_value (entry)))
@@ -1769,7 +1897,16 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 
 	toplevel_ids = gconf_value_get_list (value);
 
-	existing_toplevels = panel_toplevel_list_toplevels ();
+	existing_toplevels = NULL;
+	for (l = panel_toplevel_list_toplevels (); l; l = l->next) {
+		PanelToplevel *toplevel = l->data;
+
+		/* Attached toplevels aren't on the id list */
+		if (panel_toplevel_get_is_attached (toplevel))
+			continue;
+
+		existing_toplevels = g_slist_prepend (existing_toplevels, toplevel);
+	}
 
 	panel_profile_load_added_ids (client,
 				      PANEL_GCONF_TOPLEVELS,
@@ -1785,6 +1922,8 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 					  toplevel_ids,
 					  (PanelProfileGetIdFunc) panel_profile_get_toplevel_id,
 					  (PanelProfileDestroyFunc) panel_profile_destroy_toplevel);
+
+	g_slist_free (existing_toplevels);
 }
 
 static void
