@@ -58,7 +58,7 @@ show_uri (const char *uri, const char *mime_type, GdkScreen *screen,
 		return FALSE;
 	}
 
-	str = g_string_new (NULL);
+	str = g_string_new ("");
 
 	if (app->requires_terminal) {
 		/* FIXME: we should use their preferred terminal
@@ -138,14 +138,17 @@ recent_documents_activate_cb (EggRecentViewGtk *view, EggRecentItem *item,
 
 	if (show_uri (uri, mime_type, screen, &error) != TRUE) {
 		if (error != NULL) {
-			panel_error_dialog (screen, "cannot_open_recent_doc",
+			panel_error_dialog (screen,
+					    "cannot_open_recent_doc",
+					    _("Cannot open recently used document"),
 					    error->message);
 			g_error_free (error);
 		} else
 			panel_error_dialog (screen,
-					"cannot_open_recent_doc_unknown",
-					_("An unknown error occurred while trying"
-					  " to open %s"), uri_utf8);
+					    "cannot_open_recent_doc_unknown",
+					    _("Cannot open recently used document"),
+					    _("An unknown error occurred while trying to open %s"),
+					    uri_utf8);
 	}
 
 	g_free (uri);
@@ -162,12 +165,18 @@ panel_recent_append_documents_menu (GtkWidget *top_menu)
 	EggRecentViewGtk *view;
 	GdkScreen        *screen;
 
-	menu_item = stock_menu_item_new (_("Open Recent"),
-					 GTK_STOCK_OPEN,
-					 FALSE);
-	menu = panel_menu_new ();
+	menu_item = gtk_image_menu_item_new ();
+	setup_stock_menu_item (menu_item,
+			       panel_menu_icon_get_size (),
+			       GTK_STOCK_OPEN,
+			       _("Open Recent"),
+			       TRUE);
+	menu = panel_create_menu ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
 
+	g_signal_connect (G_OBJECT (menu), "button_press_event",
+			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+
 	gtk_menu_shell_append (GTK_MENU_SHELL (top_menu), menu_item);
 	gtk_widget_show_all (menu_item);
 	
