@@ -105,14 +105,6 @@ enum {
 
 static PanelRunDialog *static_dialog = NULL;
 
-static void
-panel_run_dialog_free (PanelRunDialog *dialog)
-{
-	if (dialog->run_dialog)
-		gtk_widget_destroy (dialog->run_dialog);
-	dialog->run_dialog = NULL;
-}
-
 static void
 panel_run_dialog_destroy (PanelRunDialog *dialog)
 {
@@ -156,6 +148,22 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 	g_free (dialog);
 }
 
+static void
+set_window_icon_from_stock (GtkWindow *window,
+			    const char *stock_id)
+{
+	GdkPixbuf *icon;
+
+	icon = gtk_widget_render_icon (GTK_WIDGET (window),
+				       stock_id,
+				       GTK_ICON_SIZE_DIALOG,
+				       NULL);
+
+	gtk_window_set_icon (window, icon);
+
+	g_object_unref (icon);
+}
+
 static void
 panel_run_dialog_set_icon (PanelRunDialog *dialog,
 			   const char     *icon_path)
@@ -183,6 +191,8 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 		 * Scaled looks worse than a smaller image.
 		 */
 		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->pixmap), pixbuf);
+
+		gtk_window_set_icon (GTK_WINDOW (dialog->run_dialog), pixbuf);
 		
 		/* Don't unref pixbuf here, GTK will do it for us. */
 		gtk_drag_source_set_icon_pixbuf (dialog->run_dialog, pixbuf);		
@@ -190,11 +200,14 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 		g_free (icon);
 		
 		gtk_image_set_from_stock (GTK_IMAGE (dialog->pixmap), 
-					  GTK_STOCK_EXECUTE,
+					  PANEL_STOCK_RUN,
 					  GTK_ICON_SIZE_DIALOG);
 		
+		set_window_icon_from_stock (GTK_WINDOW (dialog->run_dialog),
+					    PANEL_STOCK_RUN);
+		
 		gtk_drag_source_set_icon_stock (dialog->run_dialog,
-						GTK_STOCK_EXECUTE);
+						PANEL_STOCK_RUN);
 	}
 }
 
@@ -272,14 +285,11 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 				&error);
 			
 	if (!result) {
-		char *msg;
-		msg = g_strdup_printf ("<b>%s: '%s'</b>\n\n%s: %s",
-			_("Error launching command"), escaped,
-			_("Details"), error->message);
+		panel_error_dialog (screen, "cannot_spawn_command",
+				    _("Cannot launch command '%s'"),
+				    error->message,
+				    escaped);
 
-		panel_error_dialog (screen, "error_spawning_command", msg);
-
-		g_free (msg);
 		g_error_free (error);
 	}
 				
@@ -306,14 +316,11 @@ panel_run_dialog_show_url (PanelRunDialog *dialog,
 	g_strfreev (envp);
 
 	if (error) {
-		char *msg;
-		msg = g_strdup_printf ("<b>%s: '%s'</b>\n\n%s: %s",
-			_("Error displaying location"), escaped,
-			_("Details"), error->message);
+		panel_error_dialog (screen, "cannot_show_url",
+				    _("Cannot display location '%s'"),
+				    error->message,
+				    escaped);
 
-		panel_error_dialog (screen, "error_showing_url", msg);
-
-		g_free (msg);
 		g_error_free (error);
 		return FALSE;
 	}
@@ -345,13 +352,13 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 		start_screen_check ();
 
 		g_free (command);
-		panel_run_dialog_free (dialog);
+		gtk_widget_destroy (dialog->run_dialog);
 		return;
 	} else if (!strcmp (command, "gegls from outer space")) {
 		start_geginv ();
 
 		g_free (command);
-		panel_run_dialog_free (dialog);
+		gtk_widget_destroy (dialog->run_dialog);
 		return;
 	}
 		
@@ -382,7 +389,7 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 					     command);
 		
 		/* only close the dialog if we successfully showed or launched something */
-		panel_run_dialog_free (dialog);
+		gtk_widget_destroy (dialog->run_dialog);
 	}
 
 	g_free (command);
@@ -402,7 +409,7 @@ panel_run_dialog_response (PanelRunDialog *dialog,
 		panel_run_dialog_execute (dialog);
 		break;
 	case GTK_RESPONSE_CANCEL:
-		panel_run_dialog_free (dialog);
+		gtk_widget_destroy (dialog->run_dialog);
 		break;
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (run_dialog)),
@@ -1408,6 +1415,10 @@ panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
 	g_signal_connect (dialog->run_dialog, "drag_data_get",
 			  G_CALLBACK (pixmap_drag_data_get),
 			  dialog);
+
+	gtk_image_set_from_stock (GTK_IMAGE (dialog->pixmap), 
+				  PANEL_STOCK_RUN,
+				  GTK_ICON_SIZE_DIALOG);
 }
 				
 static PanelRunDialog *
@@ -1444,7 +1455,7 @@ panel_run_dialog_new (GdkScreen *screen,
 
 	gtk_window_set_screen (GTK_WINDOW (dialog->run_dialog), screen);
 
-	/* FIXME: what about window icon? */
+	set_window_icon_from_stock (GTK_WINDOW (dialog->run_dialog), PANEL_STOCK_RUN);
 
 	gtk_widget_grab_focus (dialog->gtk_entry);
 	gtk_widget_show (dialog->run_dialog);
